/// <summary>
/// Summary description for DataBaseTree control
/// User control for populating database and tables
/// </summary>
/// <author> Rajesh Pillai</author>
/// <Date> March 2004 </Date>

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;


namespace DBControls
{
	public class DataBaseTree : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.TreeView tvDataBase;
		private System.ComponentModel.IContainer components;

		public string SPText; 

		private string		_server		= "";
		private string		_database	= "";
		private string		_uid		= "";
		private bool		_isTrusted  = false;

		private ContextMenu ctxAction;
		private MenuItem	mnuOpen;
		private MenuItem	mnuEdit;
		private string		_pwd		= "";

		private string selectedDatabase;
		private System.Windows.Forms.ImageList imlTree;
		private System.Windows.Forms.MenuItem mnuRefresh;
		private string selectedTable;

		private string SELECTED_OBJECT;


		//Delegates
		delegate void OpenClick(object sender, System.EventArgs e);
		delegate void EditClick(object sender, System.EventArgs e);
		
		public event EventHandler OnSPSelected;


		public DataBaseTree()
		{
			InitializeComponent();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(DataBaseTree));
			this.tvDataBase = new System.Windows.Forms.TreeView();
			this.ctxAction = new System.Windows.Forms.ContextMenu();
			this.mnuOpen = new System.Windows.Forms.MenuItem();
			this.mnuEdit = new System.Windows.Forms.MenuItem();
			this.mnuRefresh = new System.Windows.Forms.MenuItem();
			this.imlTree = new System.Windows.Forms.ImageList(this.components);
			this.SuspendLayout();
			// 
			// tvDataBase
			// 
			this.tvDataBase.ContextMenu = this.ctxAction;
			this.tvDataBase.ImageList = this.imlTree;
			this.tvDataBase.Location = new System.Drawing.Point(6, 4);
			this.tvDataBase.Name = "tvDataBase";
			this.tvDataBase.Size = new System.Drawing.Size(232, 304);
			this.tvDataBase.TabIndex = 0;
			this.tvDataBase.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.tvDataBase_AfterSelect);
			// 
			// ctxAction
			// 
			this.ctxAction.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mnuOpen,
																					  this.mnuEdit,
																					  this.mnuRefresh});
			// 
			// mnuOpen
			// 
			this.mnuOpen.Index = 0;
			this.mnuOpen.Text = "&View";
			this.mnuOpen.Click += new System.EventHandler(this.mnuOpen_Click);
			// 
			// mnuEdit
			// 
			this.mnuEdit.Index = 1;
			this.mnuEdit.Text = "&Edit";
			this.mnuEdit.Click += new System.EventHandler(this.mnuEdit_Click);
			// 
			// mnuRefresh
			// 
			this.mnuRefresh.Index = 2;
			this.mnuRefresh.Shortcut = System.Windows.Forms.Shortcut.F5;
			this.mnuRefresh.Text = "&Refresh";
			this.mnuRefresh.Click += new System.EventHandler(this.mnuRefresh_Click);
			// 
			// imlTree
			// 
			this.imlTree.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.imlTree.ImageSize = new System.Drawing.Size(16, 16);
			this.imlTree.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imlTree.ImageStream")));
			this.imlTree.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// DataBaseTree
			// 
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.tvDataBase});
			this.Name = "DataBaseTree";
			this.Size = new System.Drawing.Size(248, 312);
			this.Resize += new System.EventHandler(this.DataBaseTree_Resize);
			this.ResumeLayout(false);

		}
		#endregion

		[Category("Connection"),Description("Specifies the server")]
		public string Server
		{
			get 
			{
				return _server;
			}
		}

		[Category("Connection"),Description("Specifies the database")]
		public string Database
		{
			get 
			{
				return _database;
			}
		}

		[Category("Connection"),Description("Specifies the username")]
		public string UserName
		{
			get 
			{
				return _uid;
			}
		}

		[Category("Connection"),Description("Specifies the password")]
		public string Password
		{
			get 
			{
				return _pwd;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="server"></param>
		/// <param name="database"></param>
		/// <param name="loginName"></param>
		/// <param name="password"></param>
		
		public void RefreshTree()
		{
			if (ValidateConnection())
				FillTree();
				
		}

		/// <summary>
		/// Fills the tree view
		/// </summary>
		/// <param name="server"></param>
		/// <param name="database"></param>
		/// <param name="loginName"></param>
		/// <param name="password"></param>
		public void FillTree(string server,string database,string loginName, string password,bool isTrusted)
		{
			if (isTrusted == false)
			{
				if (!ValidateConnection(server,database,loginName,password))
					return;
			}
			else 
			{
				if (!ValidateConnection(server,database))
					return;
			}
			
			DataTable dt;

			DataLayer.SqlDBHandler dh;
			if (isTrusted == false)
				dh = new DataLayer.SqlDBHandler(server,database,loginName,password);
			else
			{
				string conStr = "Data Source = " + server + ";Integrated Security=SSPI;Initial Catalog=" + database ;
				dh = new DataLayer.SqlDBHandler(conStr);
			}
			
			
			TreeNode tn;
			tn=FindTreeNode(tvDataBase,server);

			tvDataBase.BeginUpdate();

			if (tn == null)
			{
				tn  = new TreeNode(server,0,0);
				tvDataBase.Nodes.Add(tn);
			}
			
			tn = FindTreeNode(tvDataBase,database);
			if (tn == null)
			{
				tn  = new TreeNode(database,1,1);

				tvDataBase.Nodes[0].Nodes.Add(tn);
				
				TreeNode TABLE = new TreeNode("Tables",2,2);
				tn.Nodes.Add(TABLE);

				TreeNode VIEWS = new TreeNode("Views",3,3);
				tn.Nodes.Add(VIEWS);

				TreeNode SP = new TreeNode("Stored Procedures",4,4);
				tn.Nodes.Add(SP);

			}

			// fill the tables
			dt=  dh.GetTables();
			
			if (dt != null)
			{	
				foreach(DataRow dr in dt.Rows)
				{
					TreeNode tn1 = FindTreeNode(tvDataBase,database);
					if (tn1 != null)
					{
						TreeNode tn2 = FindTreeNode(tvDataBase,dr[0].ToString());
						if (tn2 == null)
						{
							tn1.Nodes[0].Nodes.Add(new TreeNode(dr[0].ToString(),2,2));
						}
					}
				}
			}

			// Fill the views
			dt = null;
			dt=  dh.GetViews();

			if (dt != null)
			{
				foreach(DataRow dr in dt.Rows)
				{
					TreeNode tn1 = FindTreeNode(tvDataBase,database);
					if (tn1 != null)
					{
						TreeNode tn2 = FindTreeNode(tvDataBase,dr[0].ToString());
						if (tn2 == null)
						{
							tn1.Nodes[1].Nodes.Add(new TreeNode(dr[0].ToString(),2,2));
						}
					}
				}
			}

			// Fill the SP
			dt = null;
			dt=  dh.GetSP();

			if (dt != null)
			{
				foreach(DataRow dr in dt.Rows)
				{
					TreeNode tn1 = FindTreeNode(tvDataBase,database);
					if (tn1 != null)
					{
						TreeNode tn2 = FindTreeNode(tvDataBase,dr[0].ToString());
						if (tn2 == null)
						{
							tn1.Nodes[2].Nodes.Add(new TreeNode(dr[0].ToString().Substring(0,dr[0].ToString().IndexOf(";")) ,2,2));
						}
					}
				}
			}

			tvDataBase.EndUpdate();
		}

		/// <summary>
		/// fills the tree
		/// </summary>
		private void FillTree()
		{
			if (!ValidateConnection())
				return;
			
			DataTable dt;
			DataLayer.SqlDBHandler dh = new DataLayer.SqlDBHandler(_server,_database,_uid,_pwd);
			dt=  dh.GetTables();
			
			TreeNode tn = FindTreeNode(tvDataBase,_server);
			if (tn == null)
			{
				tvDataBase.Nodes.Add(new TreeNode(_server,0,0));
			}
			tn = null;
			tn = FindTreeNode(tvDataBase,_database);
			
			if (tn == null)
			{
				tvDataBase.Nodes[0].Nodes.Add(new TreeNode(_database,1,1));
			}

			if (dt == null)
				return;
			foreach(DataRow dr in dt.Rows)
			{
				TreeNode tn1 = FindTreeNode(tvDataBase,_database);
				if (tn1 != null)
				{
					tn1.Nodes.Add(new TreeNode(dr[0].ToString(),2,2));
				}
			}
		}


		/// <summary>
		/// check for valid connection
		/// </summary>
		/// <returns></returns>
		private bool ValidateConnection()
		{
			return (_server != "" && _database != "");
		}

		private bool ValidateConnection(string server, string database)
		{
			return (server != "" && database != "");
		}

		private bool ValidateConnection(string server, string database, string uid, string pwd)
		{
			_server = server;
			_database = database;
			_uid = uid;
			_pwd = pwd;
			return (server != "" && database != "" && uid != "");
		}

		/// <summary>
		/// Search the tree for the specified node recursively
		/// </summary>
		/// <param name="treeNode"></param>
		/// <param name="NodeToFind"></param>
		/// <returns></returns>
		private TreeNode SearchRecursive(TreeNode treeNode, string NodeToFind)
		{
			foreach (TreeNode tn in treeNode.Nodes)
			{
				
				if (tn.Text == NodeToFind)
				{
					return tn;
				}
				SearchRecursive(tn,NodeToFind);
			}
			return null;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="treeView"></param>
		/// <param name="NodeToFind"></param>
		/// <returns></returns>
		
		private TreeNode FindTreeNode(TreeView treeView, string NodeToFind)
		{
			// Print each node recursively.

			TreeNodeCollection nodes = treeView.Nodes;
			foreach (TreeNode n in nodes)
			{
				if (n.Text == NodeToFind)
					return n;
				return SearchRecursive(n,NodeToFind);
				
			}
			return null;
		}

		public void ClearTree()
		{
			tvDataBase.Nodes.Clear();
			
		}

		/// <summary>
		/// Adjust the tvDatabase during resize operation
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void DataBaseTree_Resize(object sender, System.EventArgs e)
		{
			tvDataBase.Top		= this.Top		+ 5;
			tvDataBase.Left		= this.Left		+ 5;
			tvDataBase.Height	= this.Height	- 5;
			tvDataBase.Width	= this.Width	- 5;
		}

		private void mnuOpen_Click(object sender, System.EventArgs e)
		{
			OpenEditor(true);
		}

		private void mnuEdit_Click(object sender, System.EventArgs e)
		{
			OpenEditor(false);
		}

		/// <summary>
		/// Opens the editor with the specified table data in view/edit mode
		/// </summary>
		/// <param name="isReadOnly"></param>
		private void OpenEditor(bool isReadOnly )
		{
			if (SELECTED_OBJECT == "VIEW" || SELECTED_OBJECT == "TABLE")
			{
				ShowEditorForTableView(isReadOnly);
			}
			else
			{
				ShowEditorForSP();

			}
		}

		private void ShowEditorForSP()
		{
			SPText  = "";
			DataLayer.SqlDBHandler dh = new DataLayer.SqlDBHandler
					(_server, selectedDatabase,_uid, _pwd);
			
			DataTable dt; 
			dt=  dh.GetSPText(selectedTable);
			
			if (dt == null)
				return;
			
			foreach(DataRow dr in dt.Rows)
			{
				SPText  += dr[0].ToString();
			}
			
			if (OnSPSelected != null)
			{
				OnSPSelected(tvDataBase,new EventArgs());
			}
		}
		
		private void ShowEditorForTableView(bool isReadOnly)
		{
			frmTableEdit te = new frmTableEdit ();
			DataTable dt;
			DataLayer.SqlDBHandler dh = new DataLayer.SqlDBHandler(_server,selectedDatabase,_uid,_pwd);
			
			try
			{
				dt=  dh.ExecDT("select * from " + selectedTable );
				dt.TableName = selectedTable;

				te.dgTable.DataSource = dt;
				te.dgTable.ReadOnly = isReadOnly;
				te.Tag = _server + "@" +  selectedDatabase + "@" + _uid + "@" + _pwd ;	// @ delimiter
				
			}
			catch (Exception ex)
			{
				System.Diagnostics.Debug.WriteLine(ex.Message);
			}

			te.Text = "Table - " + selectedTable + ": " + (!isReadOnly?"Edit mode":"View mode");

			System.Windows.Forms.Form container;
			container = this.FindForm();

			te.MdiParent = container;

			te.Show();

		}

		/// <summary>
		/// sets the selected database and table after a node is selected
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void tvDataBase_AfterSelect(object sender, System.Windows.Forms.TreeViewEventArgs e)
		{
			try
			{
				selectedDatabase = e.Node.Parent.Parent.Text;
				// set the selected table 
				selectedTable = "[" + e.Node.Text + "]";
				mnuEdit.Enabled = true;
				
				if (e.Node.Parent.Text == "Views")
				{
					mnuEdit.Enabled = false;
					SELECTED_OBJECT = "VIEW";
				}
				else if (e.Node.Parent.Text == "Stored Procedures")
				{
					SELECTED_OBJECT = "SP";
				}
				else if (e.Node.Parent.Text == "Tables")
				{
					SELECTED_OBJECT = "TABLE";
				}

					 
			}
			catch (Exception ex)
			{
				System.Diagnostics.Debug.WriteLine(ex.Message);
			}
		}

		/// <summary>
		/// Refresh the tree view by refetching the database
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		/// TODO: need to optimise this routine.  for eg. if the table is selected
		/// and click on refresh only that table info. should be refetched.
		/// 
		private void mnuRefresh_Click(object sender, System.EventArgs e)
		{
			if(_server == selectedDatabase)
				return;
			ClearTree();
			
			QASharp.DataBase db = new QASharp.DataBase();
			DataTable dt =    db.GetDatabases(_server,_uid,_pwd,"master");
			
			foreach(DataRow dr in dt.Rows)
			{
				FillTree(_server,dr[0].ToString(),_uid, _pwd,_isTrusted);
			}

		}
	}
}
